rm(list=ls())
source("../jsz_functions.r")

dtsm.unr <- "jsz2-N4-ols"
#dtsm.unr <- "jsz2-ols"

country <- "us"

mats <- c(3, 6, 12, 24, 36, 48, 60, 72, 84, 96, 108, 120, 180, 240)
J <- length(mats)
n.per <- 252 
mats <- mats*n.per/12 
dt <- 1

file.name <- paste('zc-data/zc_', country, '.csv', sep="")
yield.data <- read.csv(file.name)

yield.data$date <- as.Date(yield.data$Date, format='%m/%d/%Y')
yield.data$dates <- as.numeric(format(yield.data$date,"%Y%m%d"))

start.date <- 19950102
end.date <- 20091231
smpl.sel <- yield.data$dates>=start.date & yield.data$dates<=end.date
yield.data <- yield.data[smpl.sel,]

date <- yield.data$date

Y <- as.matrix(yield.data[,2:(J+1)])
Y <- Y/100/n.per ## per day
T <- dim(Y)[1]

eig <- eigen(cov(Y))
#W <- t(eig$vectors[,1:N])

## unrestricted model
file.load <- paste("dtsm-estimates/results_", country, "_", dtsm.unr, ".RData", sep="")
ols.results <- load(file.load)

cP <- Y %*% t(W)

## jsz.loadings and jsz.loadings.rho0cP are not consistent
## - for a given kinfQ, the following two procedures lead to different AcP
## (A) first jsz.rotation to obtain rho0.cP, then jsz.loadings.rho0cP to obtain AcP
## (B) jsz.loadings to directly obtain AcP

K1Q.X <- diag(lamQ.hat)
adjK1QX <- jszAdjustK1QX(K1Q.X)
K1Q.X <- adjK1QX$K1Q.X
m1 <- adjK1QX$m1

K0Q.X <- matrix(0,N,1)
K0Q.X[m1] <- kinfQ.hat

Sigma.cP <- Omega.hat

RMSE <- function(AcP, BcP) {
  Yhat <- rep(1,T)%*%AcP + cP %*% BcP
  return( sqrt(sum((Y-Yhat)^2)/T/J)*n.per*10000 )
}

## (A.1)
print("*** (A.1) ***")
loads1 <- jsz.loadings(W, K1Q.X, kinfQ.hat, Sigma.cP, mats, dt)
BX <- loads1$BX
AX <- loads1$AX

params <- jsz.rotation(W, K1Q.X, K0Q.X, dt, BX, AX);
rho0.cP <- params$rho0.cP

loads2 <- jsz.loadings.rho0cP(W, K1Q.X, rho0.cP, Sigma.cP, mats, dt)

print(rho0.cP)
print(loads2$AcP)
cat("RMSE = ", RMSE(loads2$AcP, loads2$BcP), "\n")

## (A.2)
## as above but obtain AX in a different way
print("*** (A.2) ***")
rho0.cP <- 0
loads3 <- jsz.loadings.rho0cP(W, K1Q.X, rho0.cP, Sigma.cP, mats, dt)
BX <- loads3$BX
alpha0.X <- loads3$alpha0.X; alpha1.X <- loads3$alpha1.X
AX <- alpha0.X*kinfQ.hat + alpha1.X;  ## these are slightly different from (A.1)
alpha0.cP <- loads3$alpha0.cP; alpha1.cP <- loads3$alpha1.cP
AcP <- alpha0.cP*kinfQ.hat + alpha1.cP;
print(AcP)

params <- jsz.rotation(W, K1Q.X, K0Q.X, dt, BX, AX);
rho0.cP <- params$rho0.cP

loads4 <- jsz.loadings.rho0cP(W, K1Q.X, rho0.cP, Sigma.cP, mats, dt)

print(rho0.cP)
print(loads4$AcP)
cat("RMSE = ", RMSE(loads4$AcP, loads4$BcP), "\n")

## (B)
print("*** (B) ***")
print(loads1$rho0.cP)
print(loads1$AcP)

cat("RMSE = ", RMSE(loads1$AcP, loads1$BcP), "\n")
